package com.redhat.brms;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.drools.compiler.kproject.ReleaseIdImpl;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieScanner;
import org.kie.api.definition.KiePackage;
import org.kie.api.definition.rule.Rule;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Path("/")
public class RestController {
  private static final Logger log = LoggerFactory.getLogger(RestController.class);
  private static String previousKey="";
  private static KieContainer kContainer=null;
  private static KieScanner kScanner=null;
  private static KieScannerWrapper kScannerWrapper;
  
  @GET
  @Path("/isAlive")
  public Response test(){
    return Response.status(200).entity("yes").build();
  }
  
  @GET
  @Path("/test/{gav}")
  public Response test(@Context HttpServletRequest request, @PathParam(value="gav") String gav) throws JsonParseException, JsonMappingException, IOException {
    log.info("[/test] Called");
    
    // grab behavioural parameters
    String kieBaseName=request.getParameter("kieBaseName");
    String settings=request.getParameter("settings");
    boolean kScannerLogging="true".equalsIgnoreCase(request.getParameter("kScannerLogging"));
    long scanInterval=request.getParameter("scanInterval")!=null?Long.parseLong(request.getParameter("scanInterval")):1000;
    
    // initialise the settings.xml if one was provided, ensure the system property its cleared if not
    if (settings!=null){
      System.out.println("setting 'kie.maven.settings.custom' to '"+settings+"'");
      System.setProperty("kie.maven.settings.custom", settings);
    }else
      System.clearProperty("kie.maven.settings.custom");
    
    
    KieSession session=null;
    StringBuffer sb=new StringBuffer();
    try{
      KieServices ks=KieServices.Factory.get();
      
      String key=gav+kieBaseName+settings;
      
      // if it's never been init, or key is different then re-create the kContainer and kScanner
      if (kContainer==null || !key.equals(previousKey)){
        previousKey=key;
        System.out.println((kContainer==null?"C":"Rec")+"reating kContainer and kScanner for "+gav);
        kContainer=ks.newKieContainer(new ReleaseIdImpl(gav));
        
        if (null!=kScanner) kScanner.stop();
        
        kScanner=ks.newKieScanner(kContainer);
        
        if (kScannerLogging){
          // this is a Timer wrapper for the kScanner, so we can sysout when the scanNow() is called 
          // and know when it should be scanning for new kjars
          kScannerWrapper=new KieScannerWrapper(kScanner);
          kScannerWrapper.start(scanInterval);
        }else
          kScanner.start(scanInterval);
        
        System.out.println("kScanner started - now scanning every "+(scanInterval/1000) +"s");
      }else
        System.out.println("Reusing old kContainer and kScanner");
      
      
      KieBase kBase;
      System.out.println("Initialising kBase");
      if (null==kieBaseName){
        kBase=kContainer.newKieBase(ks.newKieBaseConfiguration());
      }else
        kBase=kContainer.newKieBase(kieBaseName, ks.newKieBaseConfiguration());
      
      
      session=kBase.newKieSession();
      if (null==session) throw new RuntimeException("Session from KieBase \""+kieBaseName+"\" is NULL!");
      
      System.out.println("Rules in kieContainer:");
      for(KiePackage kp:session.getKieBase().getKiePackages())
        for(Rule r:kp.getRules()){
          System.out.println(" - "+kp.getName()+".\""+r.getName()+"\"");
          sb.append(r.getName());
        }
    }finally{
      if (null!=session)
        session.dispose();
    }
    return Response.status(200).entity(sb.toString()).build();
  }
  
  
  // this is a wrapper for the kieScanner.scanNow() operation to sysout so we know when its called
  private Timer timer;
  private class KieScannerWrapper{
    private KieScanner kScanner;
    public KieScannerWrapper(KieScanner kScanner){
      this.kScanner=kScanner;
    }
    public void start(long interval){
      timer=new Timer();
      timer.schedule(new KieScannerTask(kScanner), 0, interval);
    }
  }
  
  private class KieScannerTask extends TimerTask {
    private KieScanner kScanner;
    public KieScannerTask(KieScanner kScanner){
      this.kScanner=kScanner;
    }
    @Override
    public void run() {
      System.out.println("kScanner.scanNow() called");
      kScanner.scanNow();
    }
  }
  
}
