package com.redhat.brms;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import org.apache.commons.io.IOUtils;
import org.junit.Assert;
import org.junit.Test;

public class Defect1 {
  private static final String NEXUS="http://localhost:9000/nexus";
  private static final String CLIENT="http://localhost:8081/reproducer2-client";
  private static final boolean kScannerScanNowLogging=false;
  
  @Test
  public void test() throws Exception{
    // download Nexus from here:
    //      http://www.sonatype.org/downloads/nexus-latest-bundle.zip
    // Unzip and change
    //      nexus-2.11.1-01/conf/nexus.properties -> application-port=9000
    
    System.out.println("Start Nexus by navigating to '<nexus>/bin' and running './nexus console'. Then press Enter");
    System.in.read();
    int responseCode=getResponseCode(NEXUS);
    Assert.assertTrue("Nexus is not available; response code for url ("+NEXUS+") is: "+responseCode, 200==responseCode);
    
    responseCode=getResponseCode(CLIENT+"/rest/isAlive");
    if (responseCode==200)
      System.out.println("Please stop the Client app to ensure a clean test, and then");
      
    System.out.println("Start Client app by going to the reproducer2/client folder, running the command below. Once done, press Enter");
    System.out.println("   'mvn clean package jetty:run -Djetty.port=8081'");
    System.in.read();
    responseCode=getResponseCode(CLIENT+"/rest/isAlive");
    Assert.assertTrue("reproducer2-client is not available; response code for url ("+CLIENT+"/rest/isAlive) is: "+responseCode, 200==responseCode);
    
    File ruleFile=new File("../rules/src/main/resources/test/defect-1.0.drl");
    System.out.println("Make sure the below file contains a rule named \"rule v1.0\", then press Enter");
    System.out.println("   "+ruleFile.getCanonicalPath());
    System.in.read();
    Assert.assertTrue("rule file doesnt contain 'rule v1.0'", IOUtils.toString(new FileInputStream(ruleFile)).contains("rule v1.0"));
    
    System.out.println("From the command line navigate to reproducer2-rules and run 'mvn clean deploy' to deploy the v1.0 rules to Nexus");
    System.out.println("Once the rules are deployed to Nexus, press Enter");
    System.in.read();
    
    System.out.println("Sending HTTP GET to the webapp to initialise the kContainer, kScanner and grab the v1.0 rules");
    String url="http://localhost:8081/reproducer2-client/rest/test/com.redhat.brms:reproducer2-rules:1.0-SNAPSHOT?kieBaseName=defaultKieBase&settings=target/classes/client-settings.xml&kScannerLogging="+(String.valueOf(kScannerScanNowLogging));
    String response=get(url);
    Assert.assertEquals("rule v1.0", response);
    
    System.out.println("Now, change the rule name in the file below to 'rule v1.1'. Once done, press Enter");
    System.out.println("   "+ruleFile.getCanonicalPath());
    System.in.read();
    Assert.assertTrue("rule file doesnt contain 'rule v1.1'", IOUtils.toString(new FileInputStream(ruleFile)).contains("rule v1.1"));
    
    System.out.println("From the command line navigate to reproducer2-rules and run 'mvn clean deploy' to deploy the new rules to Nexus");
    System.out.println("Once the rules are deployed to Nexus, press Enter");
    System.in.read();
    
    System.out.println("Sending another HTTP GET to the webapp to see if the client has updated to the v1.1 rules");
    String url2="http://localhost:8081/reproducer2-client/rest/test/com.redhat.brms:reproducer2-rules:1.0-SNAPSHOT?kieBaseName=defaultKieBase&settings=target/classes/client-settings.xml&kScannerLogging="+(String.valueOf(kScannerScanNowLogging));
    String response2=get(url2);
    Assert.assertEquals("rule names returned did not contain 'rule v1.1', kBase rules not updated", "rule v1.1", response2);
  }
  
  
  public int getResponseCode(String url) throws IOException{
    HttpURLConnection connection = (HttpURLConnection)new URL(url).openConnection();
    connection.setRequestMethod("GET");
    connection.connect();
    return connection.getResponseCode();
  }
  public String get(String url) throws IOException{
    HttpURLConnection connection = (HttpURLConnection)new URL(url).openConnection();
    connection.setRequestMethod("GET");
//    if (user!=null & password!=null)
//      connection.setRequestProperty("Authorization", "Basic "+javax.xml.bind.DatatypeConverter.printBase64Binary((user+":"+password).getBytes()));
    connection.connect();
    if (200!=connection.getResponseCode()) throw new RuntimeException("Error: ResponseCode was: "+connection.getResponseCode());
    return IOUtils.toString(connection.getInputStream());
  }
  
}
